from nexuscli import exception
from nexuscli.api.repository.recipes.base import Repository
from nexuscli.api.repository.recipes.base_hosted import HostedRepository
from nexuscli.api.repository.recipes.base_group import GroupRepository
from nexuscli.api.repository.recipes.base_proxy import ProxyRepository

__all__ = ['NpmHostedRepository', 'NpmProxyRepository', 'NpmGroupRepository']


class _NpmRepository(Repository):
    DEFAULT_RECIPE = 'npm'


class NpmGroupRepository(_NpmRepository, GroupRepository):
    pass


class NpmHostedRepository(_NpmRepository, HostedRepository):
    def upload_file(self, src_file, dst_dir=None, dst_file=None):
        """
        Upload a single file to a npm repository.

        :param src_file: path to the local file to be uploaded.
        :param dst_dir: NOT USED
        :param dst_file: NOT USED
        :raises exception.NexusClientInvalidRepositoryPath: invalid repository
            path.
        :raises exception.NexusClientAPIError: unknown response from Nexus API.
        """
        params = {'repository': self.name}
        files = {'npm.asset': open(src_file, 'rb').read()}

        response = self.nexus_client.http_post(
            'components', files=files, params=params, stream=True)

        if response.status_code != 204:
            raise exception.NexusClientAPIError(
                f'Uploading to {self.name}. Reason: {response.reason} '
                f'Status code: {response.status_code} Text: {response.text}')


class NpmProxyRepository(_NpmRepository, ProxyRepository):
    pass
